package io.tswf.transmutations.transmute.injection.missing.logic

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.injection.missing.IMissingTargetStrategyLogic
import io.tswf.transmutations.transmute.injection.missing.MissingTargetHandleContext
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import org.objectweb.asm.tree.InsnNode
import org.objectweb.asm.tree.MethodNode

import static org.objectweb.asm.Opcodes.*

@AutoService(IMissingTargetStrategyLogic)
@CompileStatic
class CreateNewMissingTargetStrategyLogic implements IMissingTargetStrategyLogic {

    @Override
    void handleMissingTarget(MissingTargetHandleContext context) {
        def targetMethodDesc = context.transmutationMethodMeta.targetMethodDescription
        def newTargetMethod = new MethodNode(ACC_PUBLIC, targetMethodDesc.name, targetMethodDesc.descriptor, null, null)
        def targetMethodReturnType = newTargetMethod.returnType
        newTargetMethod.instructions << targetMethodReturnType.defaultValueInsnNode
        newTargetMethod.instructions << new InsnNode(targetMethodReturnType.getOpcode(IRETURN))
        context.targetClass.methods << newTargetMethod
        context.newTargetMethod = newTargetMethod
    }

    @Override
    MissingTargetStrategy getEnumValue() {
        return MissingTargetStrategy.CREATE_NEW
    }
}
