package io.tswf.transmutations.transmute.injection.missing.logic

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.transmutations.transmute.injection.missing.IMissingTargetStrategyLogic
import io.tswf.transmutations.transmute.injection.missing.MissingTargetHandleContext
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy

@AutoService(IMissingTargetStrategyLogic)
@CompileStatic
class WarningMissingTargetStrategyLogic implements IMissingTargetStrategyLogic {

    // Загружаем самую приоритетную стратегию для логирования (нужно для окружений, где может быть любой кастомный логгер)
    @Lazy
    ITransmutationMissingLoggingProvider loggingProvider = ServiceLoader.load(ITransmutationMissingLoggingProvider)
        .toList()
            .stream()
                .reverseSortedBy { it.priority }
                .findFirst().orElseThrow { new RuntimeException("There is no transmutation logging strategies found!") }

    @Override
    void handleMissingTarget(MissingTargetHandleContext context) {
        loggingProvider.log("There is no target for transmutation ${context.transmutationMethodMeta.transmutationMethod.javaFullName} located in ${context.transmutationMethodMeta.transmutationOwner.javaName} found in target class ${context.targetClass.javaName}. Skipping...")
    }

    @Override
    MissingTargetStrategy getEnumValue() {
        return MissingTargetStrategy.WARNING
    }
}

@CompileStatic
interface ITransmutationMissingLoggingProvider {
    default int getPriority() { 0 }
    void log(String logMessage)
}

@AutoService(ITransmutationMissingLoggingProvider)
@Slf4j
@CompileStatic
class Slf4jLoggingProvider implements ITransmutationMissingLoggingProvider {

    public static final int PRIORITY = -1

    @Override
    int getPriority() {
        return PRIORITY
    }

    @Override
    void log(String logMessage) {
        log.warn(logMessage)
    }
}