
package io.tswf.transmutations.transmute.injection.parameter

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.annotation.IsReturned
import io.tswf.transmutations.transmute.annotation.IsThrown
import io.tswf.transmutations.transmute.exception.InvalidTransmutationException
import io.tswf.transmutations.transmute.injection.ITransmutationMethodCustomParameterLoader
import io.tswf.transmutations.transmute.injection.contexts.TransmutationInjectionToMethodAtInsnContext
import org.objectweb.asm.Opcodes
import org.objectweb.asm.Type
import org.objectweb.asm.tree.InsnNode

import static org.objectweb.asm.Opcodes.ICONST_0
import static org.objectweb.asm.Opcodes.ICONST_1

@AutoService(ITransmutationMethodCustomParameterLoader)
@CompileStatic
class IsReturnedCustomParameterLoader implements ITransmutationMethodCustomParameterLoader {

    @Override
    void loadCustomParameter(TransmutationInjectionToMethodAtInsnContext context, int parameterIndex, Type parameterType) {
        if (parameterType != Type.BOOLEAN_TYPE) {
            throw new InvalidTransmutationException("The annotated with @${IsReturned.simpleName} ${parameterIndex}'st parameter of ${context.allPointsContext.transmutationMetadata.transmutationMethod.javaFullName} of ${context.allPointsContext.transmutationMetadata.transmutationOwner.javaName} type must be a primitive boolean, but was ${parameterType.className}")
        }

        context.insnsToInsertToCurrent << new InsnNode(Opcodes.isOpcodeReturn(context.startInsn.opcode) ? ICONST_1 : ICONST_0)
    }
}
