package io.tswf.transmutations.transmute.injection.parameter

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.annotation.LocalVariableValue
import io.tswf.transmutations.transmute.annotation.ReturnedValue
import io.tswf.transmutations.transmute.exception.InvalidTransmutationException
import io.tswf.transmutations.transmute.injection.ITransmutationMethodCustomParameterLoader
import io.tswf.transmutations.transmute.injection.contexts.TransmutationInjectionToMethodAtInsnContext
import org.objectweb.asm.Opcodes
import org.objectweb.asm.Type
import org.objectweb.asm.tree.VarInsnNode

import static org.objectweb.asm.Opcodes.*
import static io.tswf.transmutations.transmute.injection.hook.DefaultHookConstants.*

@AutoService(ITransmutationMethodCustomParameterLoader)
@CompileStatic
class ReturnedValueCustomParameterLoader implements ITransmutationMethodCustomParameterLoader {

    @Override
    void loadCustomParameter(TransmutationInjectionToMethodAtInsnContext context, int parameterIndex, Type parameterType) {
        def transmutationMethod = context.allPointsContext.transmutationMetadata.transmutationMethod
        def targetMethod = context.allPointsContext.targetMethod
        def targetMethodReturnType = context.allPointsContext.targetMethod.returnType

        if (targetMethodReturnType == Type.VOID_TYPE) {
            throw new InvalidTransmutationException("Can't load returned value for ${parameterIndex}'st parameter with type ${parameterType.className} (annotated with @${ReturnedValue.simpleName}) of method ${transmutationMethod.javaFullName} in ${context.allPointsContext.transmutationMetadata.transmutationOwner.javaName}, because transmute target method ${targetMethod.javaFullName} in ${context.allPointsContext.targetMethodOwner.javaName} has VOID return type that can't be stored into parameter!")
        }

        if (parameterType != targetMethodReturnType) {
            throw new InvalidTransmutationException("The annotated with @${LocalVariableValue.simpleName} ${parameterIndex}'st parameter of ${context.allPointsContext.transmutationMetadata.transmutationMethod.javaFullName} of ${context.allPointsContext.transmutationMetadata.transmutationOwner.javaName} type must be a ${targetMethodReturnType.className}, but was ${parameterType.className}")
        }

        // Инъекции одной и той же трансмутации могут происходить в разные места метода, таким образом не для всех вызовов трансмутации будут значения, возвращенные методом-целью.
        if (Opcodes.isOpcodeReturn(context.startInsn.opcode)) {
            def returnStoredLocalVarIndex = (Integer) context.allPointsContext.customPayload.getByKey(RETURN_LOCAL_VAR_INDEX_KEY)
                .ensureNotNull { new InvalidTransmutationException("Can't load returned value as transmutation parameter because there is no return value storage local variable found!") }

            context.insnsToInsertToCurrent << new VarInsnNode(targetMethodReturnType.getOpcode(ILOAD), returnStoredLocalVarIndex)
        } else {
            context.insnsToInsertToCurrent << targetMethodReturnType.defaultValueInsnNode
        }
    }
}
