package io.tswf.transmutations.transmute.injection.points

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.exception.InjectionPointCantBeReachedException
import org.objectweb.asm.Type
import org.objectweb.asm.tree.AbstractInsnNode
import org.objectweb.asm.tree.VarInsnNode

import static org.objectweb.asm.Opcodes.ILOAD
import static org.objectweb.asm.Opcodes.ISTORE

@AutoService(ITransmutationInjectionPoint)
@CompileStatic
class LocalVariableCallTransmutationInjectionPoint implements ITransmutationInjectionPoint {
    int localVarIndex
    Type localVarType
    LocalVariableCallType callType

    LocalVariableCallTransmutationInjectionPoint(int localVarIndex, Type localVarType, LocalVariableCallType callType) {
        this.localVarIndex = localVarIndex
        this.localVarType = localVarType
        this.callType = callType
    }

    @Override
    AbstractInsnNode findNextNodeNearInject(
            InjectionPointSearchContext context
    ) {
        def localVarCallInsn = localVarType.getOpcode(callType == LocalVariableCallType.GET ? ILOAD : ISTORE)

        // Validate that local variable is exists and valid
        def foundLocalVar = context.transmutationTargetMethodNode.localVariables
                .find { it.index == localVarIndex }
                ?.also { foundLocalVariable ->
                    if (foundLocalVariable.desc != localVarType.descriptor) {
                        throw new InjectionPointCantBeReachedException(
                                "Local variable with index ${localVarIndex} found at ${context.transmutationTargetMethodNode.javaFullName}, but its type mismatching. Expected '${localVarType.descriptor}', but found '${foundLocalVariable.desc}'!"
                        )
                    }
                }

        if (foundLocalVar == null) {
            return null
        }

        return context.insnFromSearch.asIterable()
            .stream()
                .filter {insnCandidate ->
                    if (insnCandidate instanceof VarInsnNode) {
                        if (insnCandidate.var != localVarIndex)
                            return false

                        if (insnCandidate.opcode != localVarCallInsn)
                            return false

                        return true
                    }

                    return false
                }
                .firstOrNull()
    }
}

@CompileStatic
enum LocalVariableCallType {
    SET,
    GET
}