package io.tswf.transmutations.transmute.injection.reader

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.annotation.IsThrown
import io.tswf.transmutations.transmute.exception.InvalidTransmutationDescriptionException
import io.tswf.transmutations.transmute.injection.parameter.IsThrownCustomParameterLoader
import org.objectweb.asm.Type

@AutoService(ICustomParameterLoaderReader)
@CompileStatic
class IsThrownCustomParameterLoaderReader implements ICustomParameterLoaderReader {

    @Override
    void handleLoaderSearch(CustomParameterLoaderReadingContext readingContext) {
        def transmutationMethod = readingContext.transmutationMetadata.transmutationMethod
        if (transmutationMethod.hasParameterAnnotation(readingContext.parameterIndex, IsThrown)) {
            if (readingContext.parameterType != Type.BOOLEAN_TYPE) {
                throw new InvalidTransmutationDescriptionException("Annotated with @${IsThrown.simpleName} parameter's of ${transmutationMethod.javaFullName} of ${readingContext.transmutationMetadata.transmutationOwner.javaName} type must be primitive boolean, not a ${readingContext.parameterType.className}!")
            }

            readingContext.transmutationMetadata.customParameterLoaders << new IsThrownCustomParameterLoader()
            readingContext.completed = true
        }
    }
}
