package io.tswf.transmutations.transmute.injection.reader

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.common.reader.ClassReferenceReader
import io.tswf.transmutations.common.remapping.TransmutationRemapper
import io.tswf.transmutations.transmute.annotation.LocalVariableValue
import io.tswf.transmutations.transmute.injection.parameter.LocalVariableCustomParameterLoader
import org.objectweb.asm.tree.AnnotationNode

@AutoService(ICustomParameterLoaderReader)
@CompileStatic
class LocalVariableCustomParameterLoaderReader implements ICustomParameterLoaderReader {

    @Override
    void handleLoaderSearch(CustomParameterLoaderReadingContext readingContext) {
        def localVariableAnnotation = readingContext.transmutationMetadata
                .transmutationMethod
                .findParameterAnnotationByType(readingContext.parameterIndex, LocalVariableValue)

        if (localVariableAnnotation != null) {
            def localVariableIndex = localVariableAnnotation.getValueByName("index", Integer)
            def localVariableTypeRef = localVariableAnnotation.getValueByName("type", AnnotationNode)
            def rawLocalVariableType = ClassReferenceReader.getTypeFromClassRefAnnotation(localVariableTypeRef)
            def remappedLocalVariableType = TransmutationRemapper.mapType(rawLocalVariableType)

            readingContext.transmutationMetadata.customParameterLoaders << new LocalVariableCustomParameterLoader(localVariableIndex, remappedLocalVariableType)

            readingContext.completed = true
        }
    }
}
