package io.tswf.transmutations.transmute.injection.reader

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.common.reader.ClassReferenceReader
import io.tswf.transmutations.common.remapping.TransmutationRemapper
import io.tswf.transmutations.transmute.annotation.ReturningIntoLocalVariable
import io.tswf.transmutations.transmute.injection.returnvalue.ReturningIntoLocalVariableTransmutationReturnValueProcessor
import org.objectweb.asm.tree.AnnotationNode

@AutoService(IReturnValueProcessorReader)
@CompileStatic
class ReturningIntoLocalVariableProcessorReader implements IReturnValueProcessorReader {

    @Override
    void handleReturnValueProcessorReading(ReturnValueProcessorReadingContext readingContext) {
        def transmutationMethod = readingContext.transmutationMetadata.transmutationMethod
        def returningAnnotation = transmutationMethod.findAnnotationByType(ReturningIntoLocalVariable)
        if (returningAnnotation != null) {
            def rawLocalVarType = ClassReferenceReader.getTypeFromClassRefAnnotation(
                    returningAnnotation.getValueByName("localVariableType", AnnotationNode)
            )
            def localVarIndex = returningAnnotation.getValueByName("localVariableIndex", Integer)
            def isForceSet = returningAnnotation.getValueByName("force", Boolean) { false }

            def remappedLocalVarType = TransmutationRemapper.mapType(rawLocalVarType)

            readingContext.readedProcessors << new ReturningIntoLocalVariableTransmutationReturnValueProcessor(remappedLocalVarType, localVarIndex, isForceSet)
        }
    }
}
