package io.tswf.transmutations.transmute.injection.reader.points

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.transmute.annotation.AtLineNumber
import io.tswf.transmutations.transmute.exception.InvalidTransmutationDescriptionException
import io.tswf.transmutations.transmute.injection.points.LineNumberTransmutationInjectionPoint
import io.tswf.transmutations.transmute.injection.reader.IInjectionPointReader
import io.tswf.transmutations.transmute.injection.reader.InjectionPointReadingContext

@AutoService(IInjectionPointReader)
@CompileStatic
class LineNumberTransmutationInjectionPointReader implements IInjectionPointReader {

    public static final String ATL_LINES_PARAMETER_NAME = "lineNumbers"

    @Override
    void handleInjectionPointReading(InjectionPointReadingContext context) {
        def lineNumbers = context.annotationNodeFromRead.getValueByName(ATL_LINES_PARAMETER_NAME, int[])

        if (lineNumbers.length == 0) {
            throw new InvalidTransmutationDescriptionException("There is empty lines array on ${context.annotationNodeFromRead.desc}!")
        }

        context.result = new LineNumberTransmutationInjectionPoint(lineNumbers)
    }

    @Override
    Class getMarkerAnnotationClass() {
        return AtLineNumber
    }
}
