package io.tswf.transmutations.transmute.util

import groovy.transform.CompileStatic

import java.util.function.Supplier

interface ICustomPayload {
    Object getByKey(String key)
    void putByKey(String key, Object value)
    def <T> T computeIfAbsent(String key, Supplier<T> value)
    boolean isEmpty()
}

@CompileStatic
class CustomPayload implements ICustomPayload {
    boolean initialized = false

    @Lazy
    Map<String, Object> customPayloadData = new HashMap<>().also { initialized = true }

    @Override
    Object getByKey(String key) {
        return notInitializedYet ? null : customPayloadData[key]
    }

    @Override
    void putByKey(String key, Object value) {
        customPayloadData[key] = value
    }

    @Override
    def <T> T computeIfAbsent(String key, Supplier<T> value) {
        return (T) customPayloadData.computeIfAbsent(key) {
            value.get()
        }
    }

    @Override
    boolean isEmpty() {
        return notInitializedYet || customPayloadData.isEmpty()
    }

    boolean isNotInitializedYet() {
        return !initialized
    }
}